/******************************************************************************
*******************************************************************************
**
**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
**
**  This copyrighted material is made available to anyone wishing to use,
**  modify, copy, or redistribute it subject to the terms and conditions
**  of the GNU General Public License v.2.
**
*******************************************************************************
******************************************************************************/

#ifndef __POOL_H__
#define __POOL_H__

#if defined(__KERNEL__) || defined(_KERNEL)
#include <linux/slab.h>
#include <linux/vmalloc.h>

#define pmalloc(x) kmalloc(x, GFP_KERNEL)
#define pfree(x,y) kfree(x)

#if (BITS_PER_LONG == 64)
#define PRIu64 "lu"
#define PRId64 "ld"
#define PRIo64 "lo"
#define PRIx64 "lx"
#define PRIX64 "lX"
#define SCNu64 "lu"
#define SCNd64 "ld"
#define SCNo64 "lo"
#define SCNx64 "lx"
#define SCNX64 "lX"
#else
#define PRIu64 "Lu"
#define PRId64 "Ld"
#define PRIo64 "Lo"
#define PRIx64 "Lx"
#define PRIX64 "LX"
#define SCNu64 "Lu"
#define SCNd64 "Ld"
#define SCNo64 "Lo"
#define SCNx64 "Lx"
#define SCNX64 "LX"
#endif

#include "pool_std.h"
#include "pool_ondisk.h"
#include "pool_incore.h"
#include "abstraction.h"
#include "pool_ioctl.h"
#include "pool_sptypes.h"
#include "common.h"

#else  /* __KERNEL__ */
#include "osi_user.h"

#include "pool_std.h"
#include "pool_ondisk.h"
#include "pool_ioctl.h"
#include "pool_sptypes.h"

#endif

#include "NSP.cf"


#define BBSHIFT 		9

#define POOL_BUSY(pp) 		((pp)->p_oblk > 0)

#ifndef ASSERT
#define ASSERT(x)  \
  if (!(x)) {   \
    printk("Pool: ASSERTION FAILED: %s\n", #x); \
  }
#endif				/* ASSERT */

/* Pool flags.   These are used in the status field of pool_t structures.
 *
 * PF_INIT:	 Temp state while pool is being added/generated.
 * PF_LOWEROPEN: In-use pool's lower devices are open. Normal state.
 * PF_VALID: 	 State when pool is not being initialized.  It's cleared to
 *		 prevent two adds from interfearing. Complement of PF_INIT.
 * PF_FREE: 	 Pool is not in use and free to be assigned.
 *
 * Unused pools have the state (PF_VALID | PF_FREE). 	  	(flag=6)
 * In-use pools have the state (PF_VALID | PF_LOWEROPEN).	(flag=3)
 * Pools being initialized are (PF_INIT).			(flag=0)
 * Pools being grown have (PF_GROWING).
 * Pools that are read-only have (PF_READONLY)
 */

#define	PF_INIT			0x00000000
#define PF_LOWEROPEN		0x00000001
#define PF_VALID		0x00000002
#define	PF_FREE			0x00000004
#define	PF_GROWING		0x00000008
#define	PF_READONLY		0x00000010

#endif				/* __POOL_H__ */
