/******************************************************************************
*******************************************************************************
**
**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
**
**  This copyrighted material is made available to anyone wishing to use,
**  modify, copy, or redistribute it subject to the terms and conditions
**  of the GNU General Public License v.2.
**
*******************************************************************************
******************************************************************************/

#ifndef __POOL_INCORE_H__
#define __POOL_INCORE_H__
#include "pool_std.h"		/* needed for constants (POOL_MAX_MULTIPATH, etc) */
#include "pool_ioctl.h"		/* needed for the pool_stat_t structure */

typedef struct device {
	uint64_t d_start;	/* starting logical blk in spool */
	uint64_t d_nblks;	/* number of blocks in device */
	uint32_t d_num_paths;
	uint32_t d_num_good_paths;
	/* device number */
	kdev_t d_dev[POOL_MAX_MULTIPATH];
	rwlock_t d_rwlock;
} device_t;

typedef struct subpool {
	uint64_t sp_start;	/* lba of first block */
	uint32_t sp_len;	/* number of blocks in subpool */
	uint32_t sp_stripe;	/* stripe size for subpool */
	uint32_t sp_type;	/* type of subpool for fs */
	uint32_t sp_devs;	/* number of devices in subpool */
	device_t *sp_devlist;	/* list of devices */
} subpool_t;

typedef struct pool {
	/* read-only values: not protected, except by POOL_VALID 
	 * which prevents opening when the structure is invalid
	 * (during pool adds, first open, and final close) */

	uint64_t p_pool_id;	/* pool id (on-disk) */
	uint64_t p_capacity;	/* number of blocks in pool */
	uint32_t p_version;	/* pool version */
	uint32_t p_minor;	/* pool minor (unused in kspc) */
	uint32_t p_blksize;	/* block size of devices */
	uint32_t p_num_spools;	/* number of subpools */
	int32_t p_oblk;		/* # times opened as block dev */
	uint32_t p_flags;	/* status flags */

	/* multipath stuff: 0=none, 1=failover, 2=round-robin */
	uint32_t p_mptype;	/* type of multipath */
	int32_t p_mpstripe;	/* stripe in kB for round-robin */

	char p_name[POOL_NAME_SIZE];	/* name of pool */
	subpool_t *p_spools;	/* subpools */

	struct rw_semaphore p_rwsema;	/* read/write semaphore */
	spinlock_t p_statslock;	/* just for stats */
	pool_stat_t p_stats;	/* total io stats */
} pool_t;

#endif				/* __POOL_INCORE_H__ */
