/******************************************************************************
*******************************************************************************
**
**  Copyright (C) Sistina Software, Inc.  1997-2001  All rights reserved.
**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
**
**  This copyrighted material is made available to anyone wishing to use,
**  modify, copy, or redistribute it subject to the terms and conditions
**  of the GNU General Public License v.2.
**
*******************************************************************************
******************************************************************************/

#ifndef __gnbd_h__
#define __gnbd_h__

#ifdef MODVERSIONS
#include <linux/modversions.h>
#endif				/*  MODVERSIONS  */
#include <linux/module.h>
#include <linux/slab.h>
#include <asm/uaccess.h>
#include <linux/smp_lock.h>
#include <linux/blk.h>
#include <linux/gnbd_ioctl.h>

#ifndef TRUE
#define TRUE (1)
#endif

#ifndef FALSE
#define FALSE (0)
#endif

#define GNBD_HARDSECT 512

#define GNBD_PING_KEY 0x3b4b5b4b
/* The first four notes for Amazing Grace
   in Harmonica Tabs. Neat huh? */

typedef struct gnbd_s {
	u64 size;
	int usage;

	int readonly;
	int server_alive;

	u32 servip;
	u16 servport;

	struct socket *sock;
	struct semaphore sender_lk;	/* makes sure only one process on the
					   client is sending at a time.  */
	struct semaphore receiver_lk;	/* makes sure that the receiver can
					   startup and shutdown in peace */
	struct semaphore trans_queue_lk;	/* makes sure that only one
						   process is modifying the
						   trans_queue at a time */
	struct semaphore notify_lk;	/* makes sure that gnbd_monitor and
					   receiver synchronize their access
					   to connected */
	struct semaphore opener_lk;	/* make sure that the device can be
					   opened and closed in peace, without
					   sitting on the gnbd_devices_lk */
	struct list_head queue;
	u32 timeout;
	struct timer_list timer;
	char *server_name;
	struct task_struct *receiver_tsk;
	struct completion wait;
	struct completion receiver_wait;

	spinlock_t stat_lock;
	gnbd_stat_t stat;
	struct iovec iov[MAX_SEGMENTS + 1];
} gnbd_t;

#define GNBD_MAX 256		/* the absolute max number of devices gnbd will
				   allow. Can't be changed */

#define gnbd_cached(x) ((x)->timeout == 0)
#define gnbd_opened(x) ((x)->usage != 0)
#define gnbd_pending(x) (!list_empty(&(x)->queue))

/* The value of output corresponds to the values INFO_DISCONNECTED,
   INFO_WAITING, INFO_CONNECTED */
static __inline__ int
gnbd_connected(gnbd_t * dev)
{
	int output = 0;
	if (dev->server_alive)
		output++;
	if (dev->sock)
		output++;
	return output;
}

extern char *gnbd_device_names[GNBD_MAX];
extern int gnbd_major;
extern gnbd_t *gnbd_devices[GNBD_MAX];
extern uint64_t insmod_time;
extern struct rw_semaphore gnbd_devices_lk;
/* prototypes so function in the other file can get back to each other. */
void gnbd_request(request_queue_t * q);
void gnbd_end_request(struct request *req);
int gnbd_end_partial_request(struct request *req, u32 bytes);
void remove_request(gnbd_t * device, struct request *req);
int gnbd_get_minor(gnbd_t * dev);
void empty_trans_queue(gnbd_t * dev);
int gnbd_send_ping(gnbd_t * device);

#define VALID_MINOR_NR(x) ((x) > 0 && (x) < 256 && gnbd_devices[x] != NULL)

#endif /*__gnbd_h__*/
