/******************************************************************************
*******************************************************************************
**
**  Copyright (C) Sistina Software, Inc.  2003  All rights reserved.
**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
**
**  This copyrighted material is made available to anyone wishing to use,
**  modify, copy, or redistribute it subject to the terms and conditions
**  of the GNU General Public License v.2.
**
*******************************************************************************
******************************************************************************/

#ifndef __gnbd_ioctl_h__
#define __gnbd_ioctl_h__

#define GNBD_IOCTL_TYPE (0xeb)
#define GNBDCOMM_IOCTL_BIT (1 << (_IOC_NRBITS - 1))
#define GNBDCOMM_IOCTL_CMD(x) _IO(GNBD_IOCTL_TYPE, GNBDCOMM_IOCTL_BIT | (x))
#define GNBD_IOCTL_CMD(x) _IO(GNBD_IOCTL_TYPE, (x))

#define GNBD_STATS          GNBD_IOCTL_CMD(1)
#define GNBD_STATS_CLEAR    GNBD_IOCTL_CMD(2)

#define GNBD_DO_GET_TIME    GNBDCOMM_IOCTL_CMD(1)
#define GNBD_DO_LIST        GNBDCOMM_IOCTL_CMD(2)
#define GNBD_DO_CREATE_DEV  GNBDCOMM_IOCTL_CMD(3)
#define GNBD_DO_REMOVE_DEV  GNBDCOMM_IOCTL_CMD(4)
#define GNBD_DO_CONNECT     GNBDCOMM_IOCTL_CMD(5)
#define GNBD_DO_DISCONNECT  GNBDCOMM_IOCTL_CMD(6)
#define GNBD_DO_FORCE_CLOSE GNBDCOMM_IOCTL_CMD(7)
#define GNBD_DO_SET_IP_PORT GNBDCOMM_IOCTL_CMD(8)
#define GNBD_DO_PING        GNBDCOMM_IOCTL_CMD(9)
#define GNBD_DO_FAIL_SERV   GNBDCOMM_IOCTL_CMD(10)

#define GNBD_NEEDS_MINOR(x) \
((x) == GNBD_DO_REMOVE_DEV || (x) == GNBD_DO_CONNECT || \
 (x) == GNBD_DO_DISCONNECT || (x) == GNBD_DO_FORCE_CLOSE || \
 (x) == GNBD_DO_SET_IP_PORT || (x) == GNBD_DO_PING)

#define GNBD_ACTION(x) \
((x) == GNBD_DO_CREATE_DEV || GNBD_NEEDS_MINOR(x))

typedef struct gnbd_list_arg_s {
	void *buf;
	int size;
} gnbd_list_arg_t;

typedef struct gnbd_info_s {
	u32 ip;			/* in big endian format */
	u32 timeout;
	u16 port;		/* in cpu format */
	u8 readonly;
	u8 minor_nr;
	u8 opened;
	u8 connected;
	u8 pending;
	u8 padding;		/* so everything lines up */
	char name[256];
} gnbd_info_t;

typedef struct gnbd_stat_s {
	u64 read_count;
	u64 read_block;
	u64 write_count;
	u64 write_block;
} gnbd_stat_t;

#define INFO_DISCONNECTED (0)
#define INFO_WAITING      (1)
#define INFO_CONNECTED    (2)

#endif				/* __gnbd_ioctl_h__ */
