/******************************************************************************
*******************************************************************************
**
**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
**
**  This copyrighted material is made available to anyone wishing to use,
**  modify, copy, or redistribute it subject to the terms and conditions
**  of the GNU General Public License v.2.
**
*******************************************************************************
******************************************************************************/

/* 
* Mostly just macros and such that get used in many places.
*/

#ifndef __gnbd_macros_h__
#define __gnbd_macros_h__

#if (BITS_PER_LONG == 64)
#define PRIu64 "lu"
#define PRIx64 "lx"
#else
#define PRIu64 "Lu"
#define PRIx64 "Lx"
#endif

#define buflen(x) (strlen(x) + 1)

/*
 * Macros to help debugging
 */
#undef printd
#ifdef GNBD_DEBUG_PRINT
#define printd(fmt, args...) printk("gnbd [%s] %ld : " fmt, current->comm, (long)(current->pid) , ## args)
#else
#define printd(fmt, args...)
#endif

/* Easy way to print out errs.*/
#undef printe
#define printe(fmt, args...) printk("gnbd [%s] %ld : " fmt, current->comm, (long)(current->pid) , ## args)

#define GNBD_RETRY(do_this, until_this) \
for (;;){ \
  do { do_this; } while(0); \
  if (until_this) \
    break; \
  printe("retrying operation at %s, %u\n", __FILE__, __LINE__); \
  if (sigismember(&(current->pending.signal), SIGKILL) || \
      sigismember(&(current->pending.signal), SIGINT) || \
      sigismember(&(current->pending.signal), SIGTERM)) \
    break; \
  yield(); \
}

#define GNBD_ASSERT(x) \
do \
{ \
  if (!(x)) \
  { \
    printk("gnbd: Assertion \"%s\" failed on line %d of file %s\n", #x, \
	  __LINE__, __FILE__);  \
    panic("gnbd: Record message above and reboot.\n"); \
  } \
} \
while (0)

static __inline__ void
gnbd_daemonize(char *thread_name)
{
	unsigned long flags;

	lock_kernel();

	daemonize();

	current->tty = NULL;
	current->tty_old_pgrp = 0;

	spin_lock_irqsave(&current->sighand->siglock, flags);
	flush_signals(current);
	sigfillset(&current->blocked);
	spin_unlock_irqrestore(&current->sighand->siglock, flags);

	sprintf(current->comm, thread_name);

	unlock_kernel();
}

#endif /*__gnbd_macros_h__*/
