/******************************************************************************
*******************************************************************************
**
**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
**
**  This copyrighted material is made available to anyone wishing to use,
**  modify, copy, or redistribute it subject to the terms and conditions
**  of the GNU General Public License v.2.
**
*******************************************************************************
******************************************************************************/

/*
* This is getting rebuilt to only hold information that is needed by both
* clients and servers.
*/

#ifndef ___trans_h__
#define ___trans_h__

#include <linux/blk.h>

/******************************************************************************
 * The io Header structure and its types.
 * This gets tacked onto the beginning of every piece of data that is
 * passed through.
 */

#define MAX_BH_LEN 128
#define SEND 1
#define RECV 0

#define PROTOCOL_VERSION 1

#define gnbd_req      0x10
#define gnbd_reply    0x20
#define gnbd_error    0x30
#define gnbd_cmd_mask 0x0f
#define gnbd_read_req      (gnbd_req|0x01)
#define gnbd_read_reply  (gnbd_reply|0x01)
#define gnbd_write_req     (gnbd_req|0x02)
#define gnbd_write_reply (gnbd_reply|0x02)
#define gnbd_shutdown 0x0e
#define gnbd_ping     0x08

#define error_invalid_request 1
#define error_server_failure 2

struct trans_hdr_s {
	u64 key;
	u32 len;
	u64 offset;
	u8 err;
	u8 type;
} __attribute__ ((packed));
typedef struct trans_hdr_s trans_hdr_t;

/*
 * ok.
 * Clients send out Requests and get replies.  All client server
 * interaction is done in this maner.
 * The meaning of the len and offset fields change depending on the type.
 * The meaning of all other fields is constant.
 * So:
 *  type is the type of this header. Types are defined above.
 *  key  is for the client to match requests with replies.
 *  err  is nonzero if the server had an error processing a request.
 * For Read Requests:
 *  len  is the amount of space the client has waiting for reply
 * For Read Replies:
 *  len  is the amount of data that is following this header.
 * For Read Requests:
 *  offset is where to find that data we want.
 * For Write Requests:
 *  offset is where to put the data we're sending.
 * For other conditions the fields should be ignored. (unless I confused
 * something here)
 */

/******************************************************************************
 * This struct holds some various bits of info that we need before the
 * client can really get going.  I think that I will eventually expand this
 * to give additional information, such as blocksize, etc, 
 */
struct serv_attrib_s {
	u64 devsize;
	u8 readonly;
} __attribute__ ((packed));
typedef struct serv_attrib_s serv_attrib_t;

/******************************************************************************
 * The new login structures.  We send a lgReq to the server, and get a
 * lgRpl back.
 *
 * For the loginRequest, you need to tell the server which device you
 * want.  This is really only for servers that server multiple block
 * devices.  In most cases this will probably be ignored.
 */

struct login_req_s {
	uint64_t timestamp;
	uint16_t version;
	char devname[256];
} __attribute__ ((packed));
typedef struct login_req_s login_req_t;

struct login_reply_s {
	uint64_t devsize;
	int32_t timeout;
	uint16_t name_size;
	uint16_t version;
	uint8_t err;
	uint8_t readonly;
} __attribute__ ((packed));
typedef struct login_reply_s login_reply_t;

#ifdef __KERNEL__

struct signal_info_s {
	unsigned long flags;
	sigset_t blocked_sav;
};
typedef struct signal_info_s signal_info_t;

int __transfer_info(int cmd, struct socket *sock, void *info, int size,
		    unsigned long timeout);
int __recv_request_data(struct socket *sock, struct request *req,
			int buf_count, unsigned long timeout);
int __send_request_data(struct socket *sock, struct request *req,
			trans_hdr_t * header, struct iovec *iov,
			unsigned long timeout);
int __trans_bh_data(int cmd, struct socket *sock, struct buffer_head *bh[],
		    int num, int start_offset, int end_len, u32 total_bytes,
		    unsigned long timeout);
int __transfer_be32(int cmd, struct socket *sock, u32 * info,
		    unsigned long timeout);
char *ip_to_str(uint32_t ip, char *buf);
int get_peer_ip(struct socket *sock, u32 * ip);

int __connect_to_gnbd_server(u32 ip, u16 port, struct socket **connection,
			     int timeout);
int setup_server_sock(u16 * port, struct socket **connection, int maxlisten,
		      int reuse);
int accept_connection(struct socket *listen_sock, struct socket **new_sock);

#define beip_to_str(ip, buf) ip_to_str(be32_to_cpu(ip), (buf))

#define DEFAULT_TRANS_TIMEOUT 5

#define transfer_info(cmd, sock, info, size) \
__transfer_info(cmd, sock, info, size, 0)

#define timed_transfer_info(cmd, sock, info, size) \
__transfer_info(cmd, sock, info, size, DEFAULT_TRANS_TIMEOUT)

#define recv_request_data(sock, req, buf_count) \
__recv_request_data(sock, req, buf_count, 0)

#define timed_recv_request_data(sock, req, buf_count) \
__recv_request_data(sock, req, buf_count, DEFAULT_TRANS_TIMEOUT)

#define send_request_data(sock, req, header, iov) \
__send_request_data(sock, req, header, iov, 0)

#define timed_send_request_data(sock, req, header, iov) \
__send_request_data(sock, req, header, iov, DEFAULT_TRANS_TIMEOUT)

#define trans_bh_data(cmd, sock, bh, num, offset, end_len, total) \
__trans_bh_data(cmd, sock, bh, num, offset, end_len, total, 0)

#define timed_trans_bh_data(cmd, sock, bh, num, offset, end_len, total) \
__trans_bh_data(cmd, sock, bh, num, offset, end_len, total, \
                DEFAULT_TRANS_TIMEOUT)

#define transfer_be32(cmd, sock, info) \
__transfer_be32(cmd, sock, info, 0)

#define timed_transfer_be32(cmd, sock, info) \
__transfer_be32(cmd, sock, info, DEFAULT_TRANS_TIMEOUT)

#define connect_to_server(ip, port, connection) \
__connect_to_gnbd_server(ip, port, connection, 0)

#define timed_connect_to_server(ip, port, connection) \
__connect_to_gnbd_server(ip, port, connection, DEFAULT_TRANS_TIMEOUT)

static __inline__ void
block_signals(signal_info_t * info)
{
	spin_lock_irqsave(&current->sighand->siglock, info->flags);
	info->blocked_sav = current->blocked;
	sigfillset(&current->blocked);
	recalc_sigpending();
	spin_unlock_irqrestore(&current->sighand->siglock, info->flags);
}

static __inline__ void
unblock_signals(signal_info_t * info)
{
	spin_lock_irqsave(&current->sighand->siglock, info->flags);
	current->blocked = info->blocked_sav;
	recalc_sigpending();
	spin_unlock_irqrestore(&current->sighand->siglock, info->flags);
}

static __inline__ void
setup_blocking(signal_info_t * info)
{
	spin_lock_irqsave(&current->sighand->siglock, info->flags);
	siginitsetinv(&current->blocked,
		      (sigmask(SIGKILL) | sigmask(SIGINT) | sigmask(SIGTERM)));
	recalc_sigpending();
	spin_unlock_irqrestore(&current->sighand->siglock, info->flags);
}
#endif				/* __KERNEL__ */

#endif /*___trans_h__*/
